# This file is generated by Lemonldap::NG::Manager::Build. Don't modify it by hand
package Lemonldap::NG::Common::Conf::DefaultValues;

our $VERSION = '2.0.3';

sub defaultValues {
    return {
        'activeTimer'        => 1,
        'ADPwdExpireWarning' => 0,
        'ADPwdMaxAge'        => 0,
        'apacheAuthnLevel'   => 4,
        'applicationList'    => {
            'default' => {
                'catname' => 'Default category',
                'type'    => 'category'
            }
        },
        'authChoiceParam' => 'lmAuth',
        'authentication'  => 'Demo',
        'available2F'     => 'UTOTP,TOTP,U2F,REST,Mail2F,Ext2F,Yubikey',
        'available2FSelfRegistration'   => 'TOTP,U2F,Yubikey',
        'bruteForceProtectionMaxAge'    => 300,
        'bruteForceProtectionMaxFailed' => 3,
        'bruteForceProtectionTempo'     => 30,
        'captcha_mail_enabled'          => 1,
        'captcha_register_enabled'      => 1,
        'captcha_size'                  => 6,
        'casAccessControlPolicy'        => 'none',
        'casAuthnLevel'                 => 1,
        'checkTime'                     => 600,
        'checkUserHiddenAttributes'     => '_2fDevices _loginHistory hGroups',
        'checkUserIdRule'               => 1,
        'checkXSS'                      => 1,
        'confirmFormMethod'             => 'post',
        'cookieName'                    => 'lemonldap',
        'cspConnect'                    => '\'self\'',
        'cspDefault'                    => '\'self\'',
        'cspFont'                       => '\'self\'',
        'cspFormAction'                 => '\'self\'',
        'cspImg'                        => '\'self\' data:',
        'cspScript'                     => '\'self\'',
        'cspStyle'                      => '\'self\'',
        'dbiAuthnLevel'                 => 2,
        'dbiExportedVars'               => {},
        'demoExportedVars'              => {
            'cn'   => 'cn',
            'mail' => 'mail',
            'uid'  => 'uid'
        },
        'domain'       => 'example.com',
        'exportedVars' => {
            'UA' => 'HTTP_USER_AGENT'
        },
        'ext2fActivation'      => 0,
        'ext2fCodeActivation'  => '\\d{6}',
        'facebookAuthnLevel'   => 1,
        'facebookExportedVars' => {},
        'facebookUserField'    => 'id',
        'failedLoginNumber'    => 5,
        'formTimeout'          => 120,
        'globalStorage'        => 'Apache::Session::File',
        'globalStorageOptions' => {
            'Directory' => '/var/lib/lemonldap-ng/sessions/',
            'generateModule' =>
              'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
            'LockDirectory' => '/var/lib/lemonldap-ng/sessions/lock/'
        },
        'gpgAuthnLevel'                 => 3,
        'gpgDb'                         => '',
        'groups'                        => {},
        'handlerInternalCache'          => 15,
        'hiddenAttributes'              => '_password',
        'httpOnly'                      => 1,
        'https'                         => -1,
        'impersonationHiddenAttributes' => '_2fDevices _loginHistory',
        'impersonationIdRule'           => 1,
        'impersonationPrefix'           => 'real_',
        'impersonationRule'             => 0,
        'impersonationSkipEmptyValues'  => 1,
        'infoFormMethod'                => 'get',
        'issuerDBCASPath'               => '^/cas/',
        'issuerDBCASRule'               => 1,
        'issuerDBGetParameters'         => {},
        'issuerDBGetPath'               => '^/get/',
        'issuerDBGetRule'               => 1,
        'issuerDBOpenIDConnectPath'     => '^/oauth2/',
        'issuerDBOpenIDConnectRule'     => 1,
        'issuerDBOpenIDPath'            => '^/openidserver/',
        'issuerDBOpenIDRule'            => 1,
        'issuerDBSAMLPath'              => '^/saml/',
        'issuerDBSAMLRule'              => 1,
        'jsRedirect'                    => 0,
        'krbAuthnLevel'                 => 3,
        'krbRemoveDomain'               => 1,
        'ldapAuthnLevel'                => 2,
        'ldapBase'                      => 'dc=example,dc=com',
        'ldapExportedVars'              => {
            'cn'   => 'cn',
            'mail' => 'mail',
            'uid'  => 'uid'
        },
        'ldapGroupAttributeName'          => 'member',
        'ldapGroupAttributeNameGroup'     => 'dn',
        'ldapGroupAttributeNameSearch'    => 'cn',
        'ldapGroupAttributeNameUser'      => 'dn',
        'ldapGroupObjectClass'            => 'groupOfNames',
        'ldapPasswordResetAttribute'      => 'pwdReset',
        'ldapPasswordResetAttributeValue' => 'TRUE',
        'ldapPort'                        => 389,
        'ldapPwdEnc'                      => 'utf-8',
        'ldapSearchDeref'                 => 'find',
        'ldapServer'                      => 'ldap://localhost',
        'ldapTimeout'                     => 120,
        'ldapUsePasswordResetAttribute'   => 1,
        'ldapVersion'                     => 3,
        'linkedInAuthnLevel'              => 1,
        'linkedInFields'             => 'id,first-name,last-name,email-address',
        'linkedInScope'              => 'r_basicprofile r_emailaddress',
        'linkedInUserField'          => 'emailAddress',
        'localSessionStorage'        => 'Cache::FileCache',
        'localSessionStorageOptions' => {
            'cache_depth'        => 3,
            'cache_root'         => '/tmp',
            'default_expires_in' => 600,
            'directory_umask'    => '007',
            'namespace'          => 'lemonldap-ng-sessions'
        },
        'locationRules' => {
            'default' => 'deny'
        },
        'logoutServices'         => {},
        'macros'                 => {},
        'mail2fActivation'       => 0,
        'mail2fCodeRegex'        => '\\d{6}',
        'mailCharset'            => 'utf-8',
        'mailFrom'               => 'noreply@example.com',
        'mailSessionKey'         => 'mail',
        'mailTimeout'            => 0,
        'mailUrl'                => 'http://auth.example.com/resetpwd',
        'managerDn'              => '',
        'managerPassword'        => '',
        'max2FDevices'           => 10,
        'max2FDevicesNameLength' => 20,
        'multiValuesSeparator'   => '; ',
        'mySessionAuthorizedRWKeys' =>
          [ '_appsListOrder', '_oidcConnectedRP', '_oidcConsents' ],
        'notificationStorage'        => 'File',
        'notificationStorageOptions' => {
            'dirName' => '/var/lib/lemonldap-ng/notifications'
        },
        'notificationWildcard'                  => 'allusers',
        'notifyDeleted'                         => 1,
        'nullAuthnLevel'                        => 0,
        'oidcAuthnLevel'                        => 1,
        'oidcRPCallbackGetParam'                => 'openidconnectcallback',
        'oidcRPStateTimeout'                    => 600,
        'oidcServiceAllowAuthorizationCodeFlow' => 1,
        'oidcServiceMetaDataAuthnContext'       => {
            'loa-1' => 1,
            'loa-2' => 2,
            'loa-3' => 3,
            'loa-4' => 4,
            'loa-5' => 5
        },
        'oidcServiceMetaDataAuthorizeURI'    => 'authorize',
        'oidcServiceMetaDataBackChannelURI'  => 'blogout',
        'oidcServiceMetaDataCheckSessionURI' => 'checksession.html',
        'oidcServiceMetaDataEndSessionURI'   => 'logout',
        'oidcServiceMetaDataFrontChannelURI' => 'flogout',
        'oidcServiceMetaDataIssuer'          => 'http://auth.example.com',
        'oidcServiceMetaDataJWKSURI'         => 'jwks',
        'oidcServiceMetaDataRegistrationURI' => 'register',
        'oidcServiceMetaDataTokenURI'        => 'token',
        'oidcServiceMetaDataUserInfoURI'     => 'userinfo',
        'openIdAuthnLevel'                   => 1,
        'openIdExportedVars'                 => {},
        'openIdIDPList'                      => '0;',
        'openIdSPList'                       => '0;',
        'openIdSreg_email'                   => 'mail',
        'openIdSreg_fullname'                => 'cn',
        'openIdSreg_nickname'                => 'uid',
        'openIdSreg_timezone'                => '_timezone',
        'pamAuthnLevel'                      => 2,
        'pamService'                         => 'login',
        'passwordDB'                         => 'Demo',
        'passwordResetAllowedRetries'        => 3,
        'port'                               => -1,
        'portal'                             => 'http://auth.example.com/',
        'portalAntiFrame'                    => 1,
        'portalCheckLogins'                  => 1,
        'portalDisplayAppslist'              => 1,
        'portalDisplayChangePassword'        => '$_auth =~ /^(LDAP|DBI|Demo)$/',
        'portalDisplayLoginHistory'          => 1,
        'portalDisplayLogout'                => 1,
        'portalDisplayOidcConsents'          => '$_oidcConnectedRP',
        'portalDisplayRegister'              => 1,
        'portalErrorOnExpiredSession'        => 1,
        'portalForceAuthnInterval'           => 5,
        'portalMainLogo'           => 'common/logos/logo_llng_400px.png',
        'portalPingInterval'       => 60000,
        'portalRequireOldPassword' => 1,
        'portalSkin'               => 'bootstrap',
        'portalUserAttr'           => '_user',
        'proxyAuthnLevel'          => 2,
        'radiusAuthnLevel'         => 3,
        'randomPasswordRegexp'     => '[A-Z]{3}[a-z]{5}.\\d{2}',
        'redirectFormMethod'       => 'get',
        'registerDB'               => 'Null',
        'registerTimeout'          => 0,
        'registerUrl'              => 'http://auth.example.com/register',
        'reloadTimeout'            => 5,
        'remoteGlobalStorage' => 'Lemonldap::NG::Common::Apache::Session::SOAP',
        'remoteGlobalStorageOptions' => {
            'ns' =>
              'http://auth.example.com/Lemonldap/NG/Common/PSGI/SOAPService',
            'proxy' => 'http://auth.example.com/sessions'
        },
        'requireToken'     => 1,
        'rest2fActivation' => 0,
        'restAuthnLevel'   => 2,
        'samlAttributeAuthorityDescriptorAttributeServiceSOAP' =>
          'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/AA/SOAP;',
        'samlAuthnContextMapKerberos'                   => 4,
        'samlAuthnContextMapPassword'                   => 2,
        'samlAuthnContextMapPasswordProtectedTransport' => 3,
        'samlAuthnContextMapTLSClient'                  => 5,
        'samlEntityID'         => '#PORTAL#/saml/metadata',
        'samlIdPResolveCookie' => 'lemonldapidp',
        'samlIDPSSODescriptorArtifactResolutionServiceArtifact' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/artifact',
        'samlIDPSSODescriptorSingleLogoutServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/singleLogout;#PORTAL#/saml/singleLogoutReturn',
        'samlIDPSSODescriptorSingleLogoutServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/singleLogout;#PORTAL#/saml/singleLogoutReturn',
        'samlIDPSSODescriptorSingleLogoutServiceSOAP' =>
'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/singleLogoutSOAP;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPArtifact' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact;#PORTAL#/saml/singleSignOnArtifact;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/singleSignOn;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/singleSignOn;',
        'samlIDPSSODescriptorWantAuthnRequestsSigned' => 1,
        'samlMetadataForceUTF8'                       => 1,
        'samlNameIDFormatMapEmail'                    => 'mail',
        'samlNameIDFormatMapKerberos'                 => 'uid',
        'samlNameIDFormatMapWindows'                  => 'uid',
        'samlNameIDFormatMapX509'                     => 'mail',
        'samlOrganizationDisplayName'                 => 'Example',
        'samlOrganizationName'                        => 'Example',
        'samlOrganizationURL'        => 'http://www.example.com',
        'samlOverrideIDPEntityID'    => '',
        'samlRelayStateTimeout'      => 600,
        'samlServiceSignatureMethod' => 'RSA_SHA1',
        'samlSPSSODescriptorArtifactResolutionServiceArtifact' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/artifact',
        'samlSPSSODescriptorAssertionConsumerServiceHTTPArtifact' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact;#PORTAL#/saml/proxySingleSignOnArtifact',
        'samlSPSSODescriptorAssertionConsumerServiceHTTPPost' =>
'0;1;urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/proxySingleSignOnPost',
        'samlSPSSODescriptorAuthnRequestsSigned' => 1,
        'samlSPSSODescriptorSingleLogoutServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/proxySingleLogout;#PORTAL#/saml/proxySingleLogoutReturn',
        'samlSPSSODescriptorSingleLogoutServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/proxySingleLogout;#PORTAL#/saml/proxySingleLogoutReturn',
        'samlSPSSODescriptorSingleLogoutServiceSOAP' =>
'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/proxySingleLogoutSOAP;',
        'samlSPSSODescriptorWantAssertionsSigned' => 1,
        'securedCookie'                           => 0,
        'sfEngine'                                => '::2F::Engines::Default',
        'sfRequired'                              => 0,
        'showLanguages'                           => 1,
        'slaveAuthnLevel'                         => 2,
        'slaveExportedVars'                       => {},
        'SMTPServer'                              => '',
        'SMTPTLS'                                 => '',
        'SSLAuthnLevel'                           => 5,
        'SSLVar'                                  => 'SSL_CLIENT_S_DN_Email',
        'SSLVarIf'                                => {},
        'successLoginNumber'                      => 5,
        'timeout'                                 => 72000,
        'timeoutActivity'                         => 0,
        'timeoutActivityInterval'                 => 60,
        'totp2fActivation'                        => 0,
        'totp2fDigits'                            => 6,
        'totp2fInterval'                          => 30,
        'totp2fRange'                             => 1,
        'totp2fSelfRegistration'                  => 0,
        'totp2fUserCanRemoveKey'                  => 1,
        'twitterAuthnLevel'                       => 1,
        'twitterUserField'                        => 'screen_name',
        'u2fActivation'                           => 0,
        'u2fSelfRegistration'                     => 0,
        'u2fUserCanRemoveKey'                     => 1,
        'upgradeSession'                          => 1,
        'userControl'                             => '^[\\w\\.\\-@]+$',
        'userDB'                                  => 'Same',
        'useRedirectOnError'                      => 1,
        'useSafeJail'                             => 1,
        'utotp2fActivation'                       => 0,
        'viewerHiddenKeys'      => 'samlIDPMetaDataNodes samlSPMetaDataNodes',
        'webIDAuthnLevel'       => 1,
        'webIDExportedVars'     => {},
        'whatToTrace'           => 'uid',
        'yubikey2fActivation'   => 0,
        'yubikey2fPublicIDSize' => 12,
        'yubikey2fSelfRegistration' => 0,
        'yubikey2fUserCanRemoveKey' => 1
    };
}

1;
